@@ -22,13 +22,13 @@ cx_Oracle.Connection = connects
 
 os.environ["NLS_LANG"] = "SIMPLIFIED CHINESE_CHINA.UTF8"
 from fast_sql.utils.exception import FILEPATH_Exceptions, TYPE_Exception, MODE_Exception
-from fast_sql.fastsql.sql import to_sql as to_SQL
-from fast_sql.fastsql.sql import to_csv as to_CSV
-from fast_sql.fastsql.sql import Read_sql
+from fast_sql.fastsql.sql import ToSql
+from fast_sql.fastsql.sql import ToCsv
+from fast_sql.fastsql.sql import ReadSql
 
 os.environ["NLS_LANG"] = "SIMPLIFIED CHINESE_CHINA.UTF8"
 
-__version__ = "1.4.4"
+__version__ = "1.4.6"
 
 
 def read_sql(
@@ -46,7 +46,6 @@ def read_sql(
     columns_upper=True,
     desc="read the scheduler",
     auto_order=False,
-    is_make_dsn=False,
 ):
     """
     Read SQL query or database table into a DataFrame.
@@ -114,7 +113,7 @@ def read_sql(
         "show_progress", value="bool"
     )
 
-    Fastsql_builder = Read_sql(
+    Fastsql_builder = ReadSql(
         sql,
         con,
         thread_num=thread_num,
@@ -124,7 +123,6 @@ def read_sql(
         desc=desc,
         columns_upper=columns_upper,
         auto_order=auto_order,
-        is_make_dsn=is_make_dsn
     )
 
     return Fastsql_builder.read_sql(
@@ -234,7 +232,7 @@ def to_csv(
         "show_progress", value="bool"
     )
 
-    Fastsql_builder = to_CSV(
+    Fastsql_builder = ToCsv(
         sql,
         con,
         thread_num=thread_num,
@@ -275,7 +273,7 @@ def to_sql(
     file_path=None,
     mode="rw",
     thread_num=15,
-    thread_w=3,
+    thread_w=8,
     encoding="utf8",
     show_progress=True,
     to_columns=None,
@@ -290,10 +288,11 @@ def to_sql(
     delete_cache=True,
     data_processing=None,
     delete_sql=None,
-    desc="rsync the scheduler",
+    desc="read the scheduler",
+    write_desc="write the scheduler",
     auto_order=False,
     extra_param=None,
-    is_make_dsn=False,
+    df=None,
     **kwargs,
 ):
     """
@@ -411,7 +410,7 @@ def to_sql(
     if mode.lower() in ["wr", "rw", "r"]:
         assert isinstance(sql, str), TYPE_Exception("sql", value="str")
 
-    Fastsql_builder = to_SQL(
+    Fastsql_builder = ToSql(
         sql,
         from_db=from_db,
         to_db=to_db,
@@ -433,7 +432,8 @@ def to_sql(
         desc=desc,
         auto_order=auto_order,
         extra_param=extra_param,
-        is_make_dsn=is_make_dsn,
+        write_desc=write_desc,
+        df=df,
         **kwargs,
     )
 
