@@ -12,12 +12,11 @@ class DB_Pool:
 
     mysql_url = ["user", "password", "port", "host", "db"]
 
-    def __init__(self, con, init_num=0, num=15, encoding=None,is_make_dsn=False):
+    def __init__(self, con, init_num=0, num=15, encoding=None):
         self.con = con
         db_engine = self.get_db_api(self.con)
         self.db_type = db_engine[0]
         self.db_config = db_engine[1]
-        self.is_make_dsn = is_make_dsn
         self.config_dict = {
             "mincached": init_num,
             "maxcached": num,
@@ -65,13 +64,11 @@ class DB_Pool:
             raise DB_Exceptions("DB_CONNECT:")
 
     def make_dsn(self, dsn):
-        if self.is_make_dsn:
-            if "DESCRIPTION" in dsn.upper():
-                return dsn
-            else:
-                _dsn = re.search(r"(.+):(\d+)/(\w+)", dsn)
-                return cx_Oracle.makedsn(*_dsn.groups())
-        return dsn
+        if "DESCRIPTION" in dsn.upper():
+            return dsn
+        else:
+            _dsn = re.search(r"(.+):(\d+)/(\w+)", dsn)
+            return cx_Oracle.makedsn(*_dsn.groups())
 
     def classification(self, con):
         if "oracle" in con.driver:
@@ -104,12 +101,11 @@ class DB_Pool:
 
 
 class Sqlalchemy_Pool(DB_Pool):
-    def __init__(self, con, num=15, encoding="utf8",is_make_dsn=False):
+    def __init__(self, con, num=15, encoding="utf8"):
         self.con = con
         db_engine = self.get_db_api(self.con)
         self.db_type = db_engine[0]
         self.db_config = db_engine[1]
-        self.is_make_dsn = is_make_dsn
 
         try:
             self.db_pool = create_engine(
@@ -156,6 +152,7 @@ class Sqlalchemy_Pool(DB_Pool):
     def dispose(self):
         self.db_pool.dispose()
 
+
 def collection_error(fun):
     def hander(*args, **kwargs):
         try:
